var helpers = require("../src/helpers");

describe("Helper functions", function() {
	var base = "";
	var prev = helpers.createClassName(base);

	describe("class name generation", function() {
		it("Unique class name creation", function() {
			var next = helpers.createClassName(base);
			expect(next).not.toEqual(prev);
			prev = next;
		});
		it("Another unique class name creation", function() {
			var next = helpers.createClassName(base);
			expect(next).not.toEqual(prev);
		});
	});

	describe("color check", function() {

		it("A valid color", function() {
			expect(helpers.isColor("#AA98BB")).toEqual(true);
		});
		it("An invalid color - missing #", function() {
			expect(helpers.isColor("AA98BB")).toEqual(false);
		});
		it("An invalid color - not a color", function() {
			expect(helpers.isColor("#manci")).toEqual(false);
		});
		it("An invalid color - too match characters", function() {
			expect(helpers.isColor("#ABABABO")).toEqual(false);
		});
		it("An invalid color - illegal letters", function() {
			expect(helpers.isColor("#12PK45")).toEqual(false);
		});

	});

	describe("multicol management", function() {

		it("if not called with a multicol", function() {
			expect(helpers.manageMulticolProps("#AA98BB")).toEqual(false);
		});

		it("if multicol is reordered on mobile", function() {
			expect(helpers.manageMulticolProps({
				type: "MULTICOLUMN",
				reorderOnMobile: true,
				cols: [],
			})).toEqual(false);
		});

		var mJMulticolNH = {
			type: "MULTICOLUMN",
			reorderOnMobile: false,
			cols: [
				{
					hideOnMobile: false,
					width: 30,
					children: []
				},
				{
					hideOnMobile: false,
					width: 35,
					children: []
				},
				{
					hideOnMobile: false,
					width: 35,
					children: []
				}
			]
		};

		it("if is not reordered, but no elem is hidden!", function() {
			helpers.manageMulticolProps(mJMulticolNH);
			expect(mJMulticolNH.cols[0].diffOnMobile).toEqual(false);
			expect(mJMulticolNH.cols[1].diffOnMobile).toEqual(false);
			expect(mJMulticolNH.cols[2].diffOnMobile).toEqual(false);
		});

		var mJMulticolH = {
			type: "MULTICOLUMN",
			reorderOnMobile: false,
			cols: [
				{
					hideOnMobile: true,
					width: 30,
					children: []
				},
				{
					hideOnMobile: false,
					width: 35,
					children: []
				},
				{
					hideOnMobile: false,
					width: 35,
					children: []
				}
			]
		};

		it("if is not reordered, but one elem is hidden!", function() {
			helpers.manageMulticolProps(mJMulticolH);
			expect(mJMulticolH.cols[0].diffOnMobile).toEqual(true);
			expect(mJMulticolH.cols[1].diffOnMobile).toEqual(true);
			expect(mJMulticolH.cols[2].diffOnMobile).toEqual(true);
			expect(mJMulticolH.cols[0].width).toEqual(30);
			expect(mJMulticolH.cols[1].width).toEqual(35);
			expect(mJMulticolH.cols[2].width).toEqual(35);
			expect(mJMulticolH.cols[0].mobileWidth).toEqual(0);
			expect(mJMulticolH.cols[1].mobileWidth).toEqual(50);
			expect(mJMulticolH.cols[2].mobileWidth).toEqual(50);
		});

	});

	describe("passing width", function () {

		var paddingExample = {
			right: 2,
			left: 4
		};

		var marginExample = {
			right: 2,
			left: 4
		};

		var borderExample = {
			right: {
				width: 3
			},
			left: {
				width: 6
			}
		};

		it("if no node is provided", function() {
			expect(helpers.passWidth()).toEqual(false);
		});

		it("if no node type is provided", function() {
			expect(helpers.passWidth({foo: 3})).toEqual(false);
		});

		it("if no boxtype node is provided", function() {
			expect(helpers.passWidth({type: "TEXT"})).toEqual(false);
		});

		it("if there is no computedWith but it needs", function() {
			expect(helpers.passWidth({
				type: "BOX"
			})).toEqual(false);
		});

		it("on single FULLWIDTH_CONTAINER", function() {
			var FWNode = {
				type: "FULLWIDTH_CONTAINER",
				twoCell: false,
				leftChildren: [
					{},
					{}
				]
			};
			helpers.passWidth(FWNode);
			expect(FWNode.leftChildren[0].computedWith).toEqual(600);
			expect(FWNode.leftChildren[1].computedWith).toEqual(600);
		});

		it("on twoCell FULLWIDTH_CONTAINER", function() {
			var FWNode = {
				type: "FULLWIDTH_CONTAINER",
				twoCell: true,
				leftChildren: [
					{},
					{}
				],
				rightChildren: [
					{},
					{}
				],
			};
			helpers.passWidth(FWNode);
			expect(FWNode.leftChildren[0].computedWith).toEqual(300);
			expect(FWNode.leftChildren[1].computedWith).toEqual(300);
			expect(FWNode.rightChildren[0].computedWith).toEqual(300);
			expect(FWNode.rightChildren[1].computedWith).toEqual(300);
		});

		it("on BOX", function() {
			var BNode = {
				type: "BOX",
				computedWith: 200,
				margin: marginExample,
				padding: paddingExample,
				border: borderExample,
				children: [
					{},
					{}
				]
			};
			helpers.passWidth(BNode);
			expect(BNode.children[0].computedWith).toEqual(179);
			expect(BNode.children[1].computedWith).toEqual(179);
		});

		it("on COMPLEX", function() {
			var CNode = {
				type: "COMPLEX",
				computedWith: 100,
				root: {
					margin: marginExample,
					padding: paddingExample,
					border: borderExample,
					children: [
						{},
						{}
					]
				}
			};
			helpers.passWidth(CNode);
			expect(CNode.root.children[0].computedWith).toEqual(79);
			expect(CNode.root.children[1].computedWith).toEqual(79);
		});

		it("on MULTICOLUMN set reorderOnMobile false if computedWith is lower then 240", function() {
			var MNode = {
				type: "MULTICOLUMN",
				computedWith: 400,
				reorderOnMobile: true,
				cols: [
					{
						width: 70,
						children: [
							{},
							{}
						]
					},
					{
						width: 30,
						children: [
							{},
							{}
						]
					}
				]
			};
			helpers.passWidth(MNode);
			expect(MNode.cols[0].children[0].computedWith).toEqual(280);
			expect(MNode.cols[0].children[1].computedWith).toEqual(280);
			expect(MNode.cols[1].children[0].computedWith).toEqual(120);
			expect(MNode.cols[1].children[1].computedWith).toEqual(120);
		});

		it("on MULTICOLUMN set reorderOnMobile false if computedWith is lower then 240", function() {
			var MNode = {
				type: "MULTICOLUMN",
				computedWith: 200,
				reorderOnMobile: true,
				cols: [
					{
						width: 70,
						children: [
							{},
							{}
						]
					},
					{
						width: 30,
						children: [
							{},
							{}
						]
					}
				]
			};
			helpers.passWidth(MNode);
			expect(MNode.reorderOnMobile).toEqual(false);
		});

	});

	describe("FULLWIDTH_CONTAINER wrapping", function() {
		it("do not do nothing if the node is not a root", function() {
			var FWsROOT = {
				type: "NOTAROOT",
				children: [
					{
						type: "FULLWIDTH_CONTAINER"
					},
					{
						type: "BOX"
					}
				]
			};
			helpers.insertFullWidths(FWsROOT);
			expect(FWsROOT.children[0].type).toEqual("FULLWIDTH_CONTAINER");
			expect(FWsROOT.children[1].type).toEqual("BOX");
			expect(FWsROOT.children.length).toEqual(2);
		});

		it("if no children is present, do nothing", function() {
			var FWsROOT = {
				type: "ROOT"
			};
			helpers.insertFullWidths(FWsROOT);
			expect(FWsROOT.children).toEqual(undefined);
		});

		it("wrap FULLWIDTH_CONTAINER around non-FULLWIDTH_CONTAINER ROOT children", function() {
			var FWsROOT = {
				type: "ROOT",
				children: [
					{
						type: "FULLWIDTH_CONTAINER"
					},
					{
						type: "BOX"
					}
				]
			};
			helpers.insertFullWidths(FWsROOT);
			expect(FWsROOT.children[0].type).toEqual("FULLWIDTH_CONTAINER");
			expect(FWsROOT.children[1].type).toEqual("FULLWIDTH_CONTAINER");
			expect(FWsROOT.children[1].leftChildren[0].type).toEqual("BOX");
		});

		it("wrap FULLWIDTH_CONTAINER around non-FULLWIDTH_CONTAINER ROOT children in a GROUP children", function() {
			var FWsROOT = {
				type: "ROOT",
				children: [
					{
						type: "FULLWIDTH_CONTAINER"
					},
					{
						type: "GROUP",
						children: [
							{
								type: "FULLWIDTH_CONTAINER"
							},
							{
								type: "BOX"
							}
						]
					},
					{
						type: "BOX"
					}
				]
			};
			helpers.insertFullWidths(FWsROOT);
			expect(FWsROOT.children[0].type).toEqual("FULLWIDTH_CONTAINER");
			expect(FWsROOT.children[1].type).toEqual("GROUP");
			expect(FWsROOT.children[1].children[0].type).toEqual("FULLWIDTH_CONTAINER");
			expect(FWsROOT.children[1].children[1].type).toEqual("FULLWIDTH_CONTAINER");
			expect(FWsROOT.children[1].children[1].leftChildren[0].type).toEqual("BOX");
			expect(FWsROOT.children[2].type).toEqual("FULLWIDTH_CONTAINER");
			expect(FWsROOT.children[2].leftChildren[0].type).toEqual("BOX");
		});
	});

	describe("create v1.0.0 typography properties from old ones", function() {

		it("do nothing if typography already exist", function() {
			var originalTypography = {
				"foo": "foo"
			};
			var typoNode = {
				typography: originalTypography
			};
			helpers.handleOldTypography(typoNode);
			expect(typoNode.typography).toEqual(originalTypography);
		});

		it("create typography node", function() {
			var textNode = {
				foo: "foo"
			};
			helpers.handleOldTypography(textNode);
			expect(textNode.typography).toBeDefined();
		});

		it("copy text infos if typo info is present", function() {
			var textStyles = "foo";
			var textNode = {
				textStyles: textStyles
			};
			helpers.handleOldTypography(textNode);
			expect(textNode.typography.text).toEqual(textStyles);
		});

		it("copy h1 infos if typo info is present", function() {
			var textStyles = "foo";
			var textNode = {
				h1Styles: textStyles
			};
			helpers.handleOldTypography(textNode);
			expect(textNode.typography.h1).toEqual(textStyles);
		});

		it("copy h2 infos if typo info is present", function() {
			var textStyles = "foo";
			var textNode = {
				h2Styles: textStyles
			};
			helpers.handleOldTypography(textNode);
			expect(textNode.typography.h2).toEqual(textStyles);
		});

		it("use default typo if typo info is present", function() {
			var textStyles = "foo";
			var textNode = {
				h1Styles: textStyles
			};
			helpers.handleOldTypography(textNode);
			expect(textNode.typography.text.size).toEqual(14);
			expect(textNode.typography.text.color).toEqual("#000001");
			expect(textNode.typography.text.family).toEqual("arial");
			expect(textNode.typography.text.lineHeight).toEqual(14);
		});

	});

});
