var generateHTMLCore = require("../src/generateHTMLCore");

describe("generateHTML", function() {
	describe("with invalid params", function () {
		var generateHTML = generateHTMLCore({
			renderCss: jasmine.createSpy(),
			createElement: jasmine.createSpy()
		});
		it("missing config parameter", function() {
			expect(function() {
				generateHTML();
			}).toThrow();
		});
		it("missing mailJSON config parameter", function() {
			expect(function() {
				generateHTML({
					DOMDocument: {}
				});
			}).toThrow();
		});
		it("missing DOMDocument config parameter", function() {
			expect(function() {
				generateHTML({
					mailJSON: {}
				});
			}).toThrow();
		});
		it("missing afterBuild config parameter", function() {
			expect(function() {
				generateHTML({
					mailJSON: {}
				});
			}).toThrow();
		});
	});

	describe("with valid params", function() {
		var generateHTML;
		var fakeRenderCss = jasmine.createSpy("renderCss");
		var fakeCreateElement = jasmine.createSpy("createElement");
		var fakeAfterBuild = jasmine.createSpy("afterBuild");

		beforeEach(function() {
			generateHTML = generateHTMLCore({
				renderCss: fakeRenderCss,
				createElement: fakeCreateElement
			});
		});

		it("should hide the placeholder element if there is any", function() {
			var mockDoc = newFakeDocument();
			var elem = mockDoc.createElement("div");
			elem.id = "placeholder";
			generateHTML({
				mailJSON: {},
				DOMDocument: mockDoc,
				manageBuild: fakeAfterBuild
			});
			expect(elem.style.display).toBe("none");
		});

		it("should not hide the non-placeholder elements", function() {
			var mockDoc = newFakeDocument();
			generateHTML({
				mailJSON: {},
				DOMDocument: mockDoc,
				manageBuild: fakeAfterBuild
			});
			expect(mockDoc.getElementById("placeholder")).toBe(null);
		});

		it("remove previous rootHeader only if it is there", function() {
			var mockDoc = newFakeDocument();
			spyOn(mockDoc.body, "removeChild");
			generateHTML({
				mailJSON: {},
				DOMDocument: mockDoc,
				manageBuild: fakeAfterBuild
			});
			expect(mockDoc.body.removeChild).not.toHaveBeenCalled();

			(mockDoc.createElement("dummy")).id="rootHeader";
			generateHTML({
				mailJSON: {},
				DOMDocument: mockDoc,
				manageBuild: fakeAfterBuild
			});
			expect(mockDoc.body.removeChild).toHaveBeenCalled();
		});

		it("remove previous styleTag only if it is there", function() {
			var mockDoc = newFakeDocument();
			spyOn(mockDoc.head, "removeChild");
			generateHTML({
				DOMDocument: mockDoc,
				manageBuild: fakeAfterBuild
			});
			expect(mockDoc.head.removeChild).not.toHaveBeenCalled();

			var randomElement = mockDoc.createElement("dummy");
			randomElement.id="styleTag";
			generateHTML({
				DOMDocument: mockDoc,
				manageBuild: fakeAfterBuild
			});
			expect(mockDoc.head.removeChild).toHaveBeenCalled();

		});

		it("remove previous rootHeader only if it is there", function() {
			var mockDoc = newFakeDocument();
			spyOn(mockDoc.body, "removeChild");
			generateHTML({
				DOMDocument: mockDoc,
				manageBuild: fakeAfterBuild
			});
			expect(mockDoc.body.removeChild).not.toHaveBeenCalled();

			var randomElement = mockDoc.createElement("dummy");
			randomElement.id="rootHeader";
			generateHTML({
				DOMDocument: mockDoc,
				manageBuild: fakeAfterBuild
			});
			expect(mockDoc.body.removeChild).toHaveBeenCalled();

		});

		it("call CSS rendering", function() {
			var mockDoc = newFakeDocument();
			generateHTML({
				mailJSON: {},
				DOMDocument: mockDoc,
				manageBuild: fakeAfterBuild
			});
			expect(fakeRenderCss).toHaveBeenCalled();
		});

		it("call root create function with the correct params - no defaultBodyBackground", function() {
			var mockDoc = newFakeDocument();
			var mailJSONRoot = {
				"document": {
					"root": {},
					"generalSettings":{
						"type":"GENERAL",
						"headerContent": null,
						"background":{
							"color":"",
							"image":{
								"src": "https://images-app.edmdesigner.com/5365cd5315069b891089418a/boiwcz3suevdczmo1vpn.png",
								"position": "center",
								"repeat": "repeat",
								"id": "53997229d3f396000094caab"
							}
						}
					},
					"usedColors": [],
					"header": null,
					"footer": null
				},
				"title": "Ukmukfuk",
				"description": "Ukmukfuk"
			};
			generateHTML({
				mailJSON: mailJSONRoot,
				DOMDocument: mockDoc,
				manageBuild: fakeAfterBuild
			});
			expect(fakeCreateElement).toHaveBeenCalledWith({
				JSONnode: mailJSONRoot,
				parent: mockDoc.body,
				DOMDocument: mockDoc,
				explicitType: "rootHeader",
				manageBuild: jasmine.any(Function),
				defaultBodyBackground: false
			});
		});

		it("call root create function with the correct params - with defaultBodyBackground", function() {
			var mockDoc = newFakeDocument();
			var defaultBodyBackground = "#112233";
			var mailJSONRoot = {
				"document": {
					"root": {},
					"generalSettings":{
						"type":"GENERAL",
						"headerContent": null,
						"background":{
							"color":"",
							"image":{
								"src": "https://images-app.edmdesigner.com/5365cd5315069b891089418a/boiwcz3suevdczmo1vpn.png",
								"position": "center",
								"repeat": "repeat",
								"id": "53997229d3f396000094caab"
							}
						}
					},
					"usedColors": [],
					"header": null,
					"footer": null
				},
				"title": "Ukmukfuk",
				"description": "Ukmukfuk"
			};
			generateHTML({
				mailJSON: mailJSONRoot,
				DOMDocument: mockDoc,
				manageBuild: fakeAfterBuild,
				defaultBodyBackground: defaultBodyBackground
			});
			expect(fakeCreateElement).toHaveBeenCalledWith({
				JSONnode: mailJSONRoot,
				parent: mockDoc.body,
				DOMDocument: mockDoc,
				explicitType: "rootHeader",
				manageBuild: jasmine.any(Function),
				defaultBodyBackground: defaultBodyBackground
			});
		});
	});
});

function mockHtmlElement(tag) {
	return {
		style: {
			display: "default"
		},
		id: null,
		tag: tag,
		innerHTML: "",
		appendChild: function() {},
		removeChild: function() {},
		classList: {
			add: function() {},
			remove: function() {}
		}
	};
}

function newFakeDocument(){
	return {
		childNodes: [],
		createElement: function(tag) {
			var newElement = mockHtmlElement(tag);
			this.childNodes.push(newElement);
			return newElement;
		},
		getElementById: function(id) {
			for (var i=0; i<this.childNodes.length; i += 1) {
				if (this.childNodes[i].id === id) {
					return this.childNodes[i];
				}
			}
			return null;
		},
		body: mockHtmlElement("body"),
		head: mockHtmlElement("head"),
		title: ""
	};
}