var unitFactoryCore = require("../src/unitFactoryCore");
var extend = require("extend");

var styles = require("../src/defaultStyles");
var helpers = require("../src/helpers");

describe("unitFactory", function() {

	var mockDoc;
	var unitFactory;
	var fakeAddProps = {
		addPadding: jasmine.createSpy(),
		addMargin: jasmine.createSpy(),
		addDividerLine: jasmine.createSpy(),
		addBorderItem: jasmine.createSpy(),
		addBorder: jasmine.createSpy(),
		addTypography: jasmine.createSpy(),
		addBackground: jasmine.createSpy(),
		addBackgroundColor: jasmine.createSpy()
	};

	var fakeSubUnitFactory = {
		createLinkElem: jasmine.createSpy().and.callFake(function(){
			return mockHtmlElement("a");
		}),
		createImageElem: jasmine.createSpy().and.callFake(function(){
			return mockHtmlElement("img");
		}),
		createTextElem: jasmine.createSpy().and.callFake(function(){
			return mockHtmlElement("div");
		})
	};

	var fakeManageBuild = jasmine.createSpy("manageBuild");

	beforeEach(function() {
		unitFactory = unitFactoryCore({
			extend: extend,
			styles: styles,
			addProps: fakeAddProps,
			helpers: helpers,
			subUnitFactory: fakeSubUnitFactory
		});
		mockDoc = newFakeDocument();
		spyOn(mockDoc, "createElement").and.callThrough();
	});

	describe("createRoot", function() {

		it("set background if background is given", function() {
			var mJRootHeader = {
				"document": {
					"root": {
						"type": "dummyRoot"
					},
					"generalSettings":{
						"type":"GENERAL",
						"headerContent":null,
						"background":{
							"color":"#AABBCC",
							"image":{
								"src":"",
								"position":"center",
								"repeat":"repeat",
								"id":""
							}
						}
					},
					"usedColors": [],
					"header": null,
					"footer": null
				},
				title: "foo",
				description: "foofoo"
			};
			unitFactory.createRoot(mJRootHeader, mockDoc);
			expect(styles.global.body.background)
			.toEqual("#AABBCC");
		});

		it("set background to white if background is not given", function() {
			var mJRootHeader = {
				"document": {
					"root": {
						"type": "dummyRoot"
					},
					"generalSettings":{
						"type":"GENERAL",
						"headerContent":null,
						"background":{
							"color":"",
							"image":{
								"src":"",
								"position":"center",
								"repeat":"repeat",
								"id":""
							}
						}
					},
					"usedColors": [],
					"header": null,
					"footer": null
				},
				title: "foo",
				description: "foofoo"
			};
			unitFactory.createRoot(mJRootHeader, mockDoc);
			expect(styles.global.body.background)
			.toEqual("#FFFFFF");
		});

		it("set title, id, and run createElement", function() {
			var mJRootHeader = {
				"document": {
					"root": {
						"type": "dummyRoot"
					},
					"generalSettings":{
						"type":"GENERAL",
						"headerContent":null,
						"background":{
							"color":"",
							"image":{
								"src":"",
								"position":"center",
								"repeat":"repeat",
								"id":""
							}
						}
					},
					"usedColors": [],
					"header": null,
					"footer": null
				},
				title: "foo",
				description: "foofoo"
			};
			var newElem = unitFactory.createRoot(mJRootHeader, mockDoc);
			expect(mockDoc.title)
			.toEqual("foo");
			expect(newElem.id)
			.toEqual("rootHeader");
			expect(mockDoc.createElement)
			.toHaveBeenCalledWith("div");
		});

	});

	describe("createOneCellFullWidth", function() {

		it("create the correct elements", function() {
			var mJFullWidth = {
				type: "FULLWIDTH_CONTAINER",
				order: "LTR",
				twoCell: false,
				leftBackgroundColor: "#AABBAA",
				rightBackgroundColor: "",
				leftChildren: [],
				rightChildren: [],
				isLeftHigher: false,
				hideOnMobile: false
			};
			var newElems = unitFactory.createFullWidthContainer(mJFullWidth, mockDoc);
			// spyOn(mockDoc, "createElement").and.callThrough();
			expect(mockDoc.createElement)
			.toHaveBeenCalledWith("div");
			expect(mockDoc.createElement.calls.count()).toEqual(2);
			expect(newElems.inner.classList.add)
			.toHaveBeenCalledWith("full-width-inner");
			expect(newElems.outer.classList.add)
			.toHaveBeenCalledWith("full-width-wrapper");
			expect(newElems.outer.appendChild)
			.toHaveBeenCalledWith(newElems.inner);
			expect(fakeAddProps.addBackgroundColor)
			.toHaveBeenCalledWith(newElems.outer, mJFullWidth.leftBackgroundColor);
		});

	});

	describe("create two cell fullwidth", function() {

		it("create the outer wrapper", function() {
			var mJFullWidth = {
				type: "FULLWIDTH_CONTAINER",
				order: "LTR",
				twoCell: true,
				leftBackgroundColor: "#AABBAA",
				rightBackgroundColor: "#CCCCCC",
				leftChildren: [],
				rightChildren: [],
				isLeftHigher: false,
				hideOnMobile: false
			};

			var newElem = unitFactory.createTwoCellFullWidthContainer(mJFullWidth, mockDoc);
			// spyOn(mockDoc, "createElement").and.callThrough();
			expect(mockDoc.createElement)
			.toHaveBeenCalledWith("div");
			expect(newElem.classList.add)
			.not.toHaveBeenCalledWith("hide-on-tablet");
		});

		it("addd hide class if hide on mobile is on", function() {
			var mJFullWidth = {
				type: "FULLWIDTH_CONTAINER",
				order: "LTR",
				twoCell: true,
				leftBackgroundColor: "#AABBAA",
				rightBackgroundColor: "#CCCCCC",
				leftChildren: [],
				rightChildren: [],
				isLeftHigher: false,
				hideOnMobile: true
			};

			var newElem = unitFactory.createTwoCellFullWidthContainer(mJFullWidth, mockDoc);
			// spyOn(mockDoc, "createElement").and.callThrough();
			expect(mockDoc.createElement)
			.toHaveBeenCalledWith("div");
			expect(newElem.classList.add)
			.toHaveBeenCalledWith("hide-on-tablet");
		});

		it("create left fullwidth part", function() {
			var mJFullWidth = {
				type: "FULLWIDTH_CONTAINER",
				order: "LTR",
				twoCell: true,
				leftBackgroundColor: "#AABBAA",
				rightBackgroundColor: "#CCCCCC",
				leftChildren: [],
				rightChildren: [],
				isLeftHigher: false,
				hideOnMobile: true
			};

			var leftElem = unitFactory.createLeftFullWidthContainer(mJFullWidth, mockDoc);
			// spyOn(mockDoc, "createElement").and.callThrough();
			expect(mockDoc.createElement)
			.toHaveBeenCalledWith("div");
			expect(mockDoc.createElement.calls.count()).toEqual(2);
			expect(leftElem.inner.classList.add)
			.toHaveBeenCalledWith("left-full-width-inner");
			expect(leftElem.outer.classList.add)
			.toHaveBeenCalledWith("left-full-width-wrapper");
			expect(leftElem.outer.appendChild)
			.toHaveBeenCalledWith(leftElem.inner);
			expect(fakeAddProps.addBackgroundColor)
			.toHaveBeenCalledWith(leftElem.outer, mJFullWidth.leftBackgroundColor);
		});

		it("create right fullwidth part", function() {
			var mJFullWidth = {
				type: "FULLWIDTH_CONTAINER",
				order: "LTR",
				twoCell: true,
				leftBackgroundColor: "#AABBAA",
				rightBackgroundColor: "#CCCCCC",
				leftChildren: [],
				rightChildren: [],
				isLeftHigher: false,
				hideOnMobile: true
			};

			var rightElem = unitFactory.createRightFullWidthContainer(mJFullWidth, mockDoc);
			// spyOn(mockDoc, "createElement").and.callThrough();
			expect(mockDoc.createElement)
			.toHaveBeenCalledWith("div");
			expect(mockDoc.createElement.calls.count()).toEqual(2);
			expect(rightElem.inner.classList.add)
			.toHaveBeenCalledWith("right-full-width-inner");
			expect(rightElem.outer.classList.add)
			.toHaveBeenCalledWith("right-full-width-wrapper");
			expect(rightElem.outer.appendChild)
			.toHaveBeenCalledWith(rightElem.inner);
			expect(fakeAddProps.addBackgroundColor)
			.toHaveBeenCalledWith(rightElem.outer, mJFullWidth.rightBackgroundColor);
		});

	});

	describe("create GROUP", function() {

		it("create the correct elements", function() {
			var mJGroupName = {
				type: "GROUP",
				name: "group-test",
				children: []
			};
			var newElems = unitFactory.createGroup(mJGroupName, mockDoc);
			// spyOn(mockDoc, "createElement").and.callThrough();
			expect(mockDoc.createElement)
			.toHaveBeenCalledWith("div");
			expect(mockDoc.createElement.calls.count()).toEqual(1);
			expect(newElems.classList.add)
			.toHaveBeenCalledWith("elem-group", "elem-inspector");
			expect(newElems.classList.add)
			.toHaveBeenCalledWith("elem-group-group-test");
		});

		it("create the correct elements without name", function() {
			var mJGroupWOName = {
				type: "GROUP",
				children: []
			};
			var newElems = unitFactory.createGroup(mJGroupWOName, mockDoc);
			// spyOn(mockDoc, "createElement").and.callThrough();
			expect(mockDoc.createElement)
			.toHaveBeenCalledWith("div");
			expect(mockDoc.createElement.calls.count()).toEqual(1);
			expect(newElems.classList.add)
			.toHaveBeenCalledWith("elem-group", "elem-inspector");
		});

		it("set hide on hideOnMobile", function() {
			var mJBox = {
				type: "BOX",
				padding: {},
				margin: {},
				border: {},
				hideOnMobile: true,
				hideOnDesktop: false,
				children: []
			};
			var newElems = unitFactory.createBox(mJBox, mockDoc);
			expect(newElems.outer.classList.add)
			.toHaveBeenCalledWith("hide-on-tablet");
			expect(newElems.outer.classList.add)
			.not.toHaveBeenCalledWith("hide-on-desktop");
		});

	});

	describe("create Box", function() {

		it("create the correct elements", function() {
			var mJBox = {
				type: "BOX",
				padding: {},
				margin: {},
				border: {},
				hideOnMobile: false,
				hideOnDesktop: false,
				children: []
			};
			var newElems = unitFactory.createBox(mJBox, mockDoc);
			// spyOn(mockDoc, "createElement").and.callThrough();
			expect(mockDoc.createElement)
			.toHaveBeenCalledWith("div");
			expect(mockDoc.createElement.calls.count()).toEqual(2);
			expect(newElems.outer.classList.add)
			.toHaveBeenCalledWith("elem-box", "elem-inspector");
			expect(newElems.outer.appendChild)
			.toHaveBeenCalledWith(newElems.inner);
			expect(fakeAddProps.addBackground)
			.toHaveBeenCalledWith(newElems.inner, mJBox.background);
			expect(fakeAddProps.addPadding)
			.toHaveBeenCalledWith(newElems.inner, mJBox.padding);
			expect(fakeAddProps.addBorder)
			.toHaveBeenCalledWith(newElems.inner, mJBox.border);
			expect(fakeAddProps.addPadding)
			.toHaveBeenCalledWith(newElems.outer, mJBox.margin);
			expect(newElems.outer.classList.add)
			.not.toHaveBeenCalledWith("hide-on-tablet");
			expect(newElems.outer.classList.add)
			.not.toHaveBeenCalledWith("hide-on-desktop");
		});

		it("set hide on hideOnDesktop", function() {
			var mJBox = {
				type: "BOX",
				padding: {},
				margin: {},
				border: {},
				hideOnMobile: false,
				hideOnDesktop: true,
				children: []
			};
			var newElems = unitFactory.createBox(mJBox, mockDoc);
			expect(newElems.outer.classList.add)
			.not.toHaveBeenCalledWith("hide-on-tablet");
			expect(newElems.outer.classList.add)
			.toHaveBeenCalledWith("hide-on-desktop");
		});

		it("set hide on hideOnMobile", function() {
			var mJBox = {
				type: "BOX",
				padding: {},
				margin: {},
				border: {},
				hideOnMobile: true,
				hideOnDesktop: false,
				children: []
			};
			var newElems = unitFactory.createBox(mJBox, mockDoc);
			expect(newElems.outer.classList.add)
			.toHaveBeenCalledWith("hide-on-tablet");
			expect(newElems.outer.classList.add)
			.not.toHaveBeenCalledWith("hide-on-desktop");
		});

	});

	describe("create textBox", function() {
		it("create the correct elements", function() {
			var mJTextBox = {
				type: "TEXT",
				spacing: {},
				typography: {},
				text: "foooo"
			};
			var newElem = unitFactory.createText(mJTextBox, mockDoc);
			expect(fakeSubUnitFactory.createTextElem)
			.toHaveBeenCalledWith(mJTextBox.text, mockDoc);
			expect(newElem.classList.add)
			.toHaveBeenCalledWith("elem-textbox", "elem-inspector");
			expect(fakeAddProps.addPadding)
			.toHaveBeenCalledWith(newElem, mJTextBox.spacing);
			expect(fakeAddProps.addTypography)
			.toHaveBeenCalledWith(newElem, mJTextBox.text, mJTextBox.typography);
		});
	});

	describe("create createImage", function() {
		var paddingExample = {
			top: 1,
			right: 2,
			bottom: 3,
			left: 4
		};

		it("create the correct elements", function() {
			var mJImage = {
				type: "IMAGE",
				padding: paddingExample,
				margin: {},
				border: {},
				background: {},
				width: 112,
				height: 100,
				align: "left",
				originalWidth: 112,
				originalHeight: 100,
				fullWidthOnMobile: false,
				src: "source",
				altText: "",
				link: "",
				sizeType: "",
				mini: false
			};
			var newElem = unitFactory.createImage(mJImage, mockDoc, fakeManageBuild);
			expect(mockDoc.createElement)
			.toHaveBeenCalledWith("div");
			expect(fakeSubUnitFactory.createImageElem)
			.toHaveBeenCalledWith(mJImage.src, mJImage.altText, mJImage.align, mockDoc, fakeManageBuild);
			expect(fakeSubUnitFactory.createLinkElem)
			.not.toHaveBeenCalled();
			expect(mockDoc.createElement.calls.count()).toEqual(2);
			expect(newElem.classList.add)
			.toHaveBeenCalledWith("elem-image", "elem-inspector");
			expect(fakeAddProps.addPadding)
			.toHaveBeenCalledWith(newElem, mJImage.margin);
			expect(styles.desktop[".im106x96"]["max-width"]).toEqual("106px");
			expect(styles.desktop[".im106x96"]["max-height"]).toEqual("96px");
			expect(styles.desktop[".im106x96"].width).toEqual("100%");
			expect(styles.tablet[".im106x96"].width).toEqual("100%");
			expect(styles.tablet[".im106x96"]["max-width"]).toEqual("106px");
		});

		it("add link if present", function() {
			var mJImage = {
				type: "IMAGE",
				padding: paddingExample,
				margin: {},
				border: {},
				background: {},
				width: 112,
				height: 100,
				align: "left",
				originalWidth: 112,
				originalHeight: 100,
				fullWidthOnMobile: false,
				src: "source",
				altText: "",
				link: "http://foo",
				sizeType: "FIXED",
				mini: false
			};
			unitFactory.createImage(mJImage, mockDoc, fakeManageBuild);
			expect(mockDoc.createElement)
			.toHaveBeenCalledWith("div");
			expect(mockDoc.createElement.calls.count()).toEqual(2);
			expect(fakeSubUnitFactory.createLinkElem)
			.toHaveBeenCalledWith(mJImage.link, mockDoc);
		});

		it("add alt text if present", function() {
			var mJImage = {
				type: "IMAGE",
				padding: paddingExample,
				margin: {},
				border: {},
				background: {},
				width: 112,
				height: 100,
				align: "left",
				originalWidth: 112,
				originalHeight: 100,
				fullWidthOnMobile: false,
				src: "source",
				altText: "KUKUCSIN",
				link: "",
				sizeType: "FIXED",
				mini: false
			};
			unitFactory.createImage(mJImage, mockDoc, fakeManageBuild);
			expect(styles.tablet[".im106x96"].width).toEqual("100%");
			expect(styles.tablet[".im106x96-full-width-on-mobile"]).toEqual(undefined);
			expect(fakeSubUnitFactory.createImageElem)
			.toHaveBeenCalledWith(mJImage.src, mJImage.altText, mJImage.align, mockDoc, fakeManageBuild);
		});

		it("if fullwidth on mobile", function() {
			var mJImage = {
				type: "IMAGE",
				padding: paddingExample,
				margin: {},
				border: {},
				background: {},
				width: 112,
				height: 200,
				align: "left",
				originalWidth: 112,
				originalHeight: 100,
				fullWidthOnMobile: true,
				src: "source",
				altText: "KUKUCSIN",
				link: "",
				sizeType: "FIXED",
				mini: false
			};
			unitFactory.createImage(mJImage, mockDoc, fakeManageBuild);
			expect(styles.tablet[".im106x196-full-width-on-mobile"]["max-width"]).toEqual(undefined);
			expect(styles.tablet[".im106x196"]).toEqual(undefined);
			expect(styles.tablet[".im106x196-full-width-on-mobile"].width).toEqual("100%");
		});

	});

	describe("create buttons", function() {

		it("create the proper elements", function() {
			var mJButton = {
				type: "BUTTON",
				padding: {},
				margin: {},
				border: {},
				background: {},
				width: 200,
				fullWidthOnMobile: false,
				align: "left",
				text: "fooooo",
				typography: {},
				href: "http://example.org",
				sizeType: "FIT_TO_TEXT",
			};
			var newElem = unitFactory.createButton(mJButton, mockDoc);
			expect(mockDoc.createElement)
			.toHaveBeenCalledWith("div");
			expect(fakeSubUnitFactory.createLinkElem)
			.toHaveBeenCalledWith(mJButton.href, mockDoc);
			expect(fakeSubUnitFactory.createTextElem)
			.toHaveBeenCalledWith(mJButton.text, mockDoc);
			expect(mockDoc.createElement.calls.count()).toEqual(1);
			expect(newElem.classList.add)
			.toHaveBeenCalledWith("elem-button", "elem-inspector");
			expect(fakeAddProps.addPadding)
			.toHaveBeenCalledWith(newElem, mJButton.margin);
			expect(styles.desktop[".btn200"]).toEqual(undefined);

		});

		it("if hideOnMobile is true", function() {
			var mJButton = {
				type: "BUTTON",
				padding: {},
				margin: {},
				border: {},
				background: {},
				width: 200,
				fullWidthOnMobile: true,
				align: "left",
				text: "fooooo",
				typography: {},
				href: "http://example.org",
				sizeType: "FIT_TO_TEXT",
			};
			var newElem = unitFactory.createButton(mJButton, mockDoc);
			expect(styles.global[".btn200"]).toEqual(undefined);
			expect(newElem.classList.add)
			.toHaveBeenCalledWith("full-width-button");
		});

		it("if sizeType is FIXED", function() {
			var mJButton = {
				type: "BUTTON",
				padding: {},
				margin: {},
				border: {},
				background: {},
				width: 200,
				fullWidthOnMobile: true,
				align: "left",
				text: "fooooo",
				typography: {},
				href: "http://example.org",
				sizeType: "FIXED",
			};
			unitFactory.createButton(mJButton, mockDoc);
			expect(styles.global[".btn200"].width).toEqual("200px");
		});

	});

	describe("create MULTICOLUMN", function() {

		it("create the proper element without hiding it on tablet", function() {
			var mJMulticol = {
				type: "MULTICOLUMN",
				hideOnMobile: false,
				reorderOnMobile: false,
				cols: []
			};
			var newElem = unitFactory.createMulticolumn(mJMulticol, mockDoc);
			expect(mockDoc.createElement)
			.toHaveBeenCalledWith("div");
			expect(mockDoc.createElement.calls.count()).toEqual(1);
			expect(newElem.classList.add)
			.not.toHaveBeenCalled();
		});

		it("test hiding on tablet", function() {
			var mJMulticol = {
				type: "MULTICOLUMN",
				hideOnMobile: true,
				reorderOnMobile: false,
				cols: []
			};
			var newElem = unitFactory.createMulticolumn(mJMulticol, mockDoc);
			expect(newElem.classList.add)
			.toHaveBeenCalledWith("hide-on-tablet");
		});

	});

	describe("create COLUMN", function() {

		it("create the proper element", function() {
			var mJCol = {
				hideOnMobile: false,
				reorderOnMobile: false,
				diffOnMobile: false,
				width: 30,
				cols: []
			};
			var newElem = unitFactory.createColumn(mJCol, mockDoc);
			expect(mockDoc.createElement)
			.toHaveBeenCalledWith("div");
			expect(mockDoc.createElement.calls.count()).toEqual(1);
			expect(newElem.classList.add)
			.toHaveBeenCalledWith("column");
			expect(newElem.classList.add)
			.toHaveBeenCalledWith("elem-column", "elem-inspector");
			expect(newElem.classList.add)
			.not.toHaveBeenCalledWith("hide-on-tablet");
			expect(styles.global[".col" + mJCol.width + "fixed"].width).toEqual("30%");
		});

		it("hideOnMobile if it is set.", function() {
			var mJCol = {
				hideOnMobile: true,
				reorderOnMobile: false,
				diffOnMobile: false,
				width: 30,
				cols: []
			};
			var newElem = unitFactory.createColumn(mJCol, mockDoc);
			expect(newElem.classList.add)
			.toHaveBeenCalledWith("hide-on-tablet");
		});

		it("reorder on mobile if it is set.", function() {
			var mJCol = {
					width: 25,
					hideOnMobile: false,
					reorderOnMobile: true,
					diffOnMobile: false,
					children: []
			};
			unitFactory.createColumn(mJCol, mockDoc);
			var classNameD = ".col" + mJCol.width + "reorder";
			expect(styles.tablet[classNameD].width).toEqual("100%");
			expect(styles.desktop[classNameD].width).toEqual("25%");
		});

		it("if one col was hidden on mobile, so diffOnMobile is set", function() {
			var mJCol = {
					width: 25,
					mobileWidth: 30,
					hideOnMobile: false,
					reorderOnMobile: false,
					diffOnMobile: true,
					children: []
			};
			unitFactory.createColumn(mJCol, mockDoc);
			var classNameD = ".col" + mJCol.width + "fixed-diff";
			expect(styles.tablet[classNameD].width).toEqual("30%");
			expect(styles.desktop[classNameD].width).toEqual("25%");
		});

	});

	describe("create Divider", function() {

		it("create the proper element", function() {
			var mJDivider = {
				type: "DIVIDER",
				background: {},
				spacing: {},
				lineStyle: {},
			};
			var newElem = unitFactory.createDivider(mJDivider, mockDoc);
			expect(mockDoc.createElement)
			.toHaveBeenCalledWith("div");
			expect(mockDoc.createElement.calls.count()).toEqual(1);
			expect(newElem.classList.add)
			.toHaveBeenCalledWith("elem-divider", "elem-inspector");
			expect(fakeAddProps.addBackground)
			.toHaveBeenCalledWith(newElem, mJDivider.background);
			expect(fakeAddProps.addMargin)
			.toHaveBeenCalledWith(newElem, mJDivider.spacing);
			expect(fakeAddProps.addDividerLine)
			.toHaveBeenCalledWith(newElem, mJDivider.lineStyle);
		});

	});

});

function mockHtmlElement(tag) {
	return {
		style: {
			display: "default"
		},
		id: null,
		tag: tag,
		innerHTML: "",
		appendChild: jasmine.createSpy(),
		removeChild: jasmine.createSpy(),
		classList: {
			add: jasmine.createSpy(),
			remove: jasmine.createSpy()
		}
	};
}

function newFakeDocument(){
	return {
		childNodes: [],
		createElement: function(tag) {
			var newElement = mockHtmlElement(tag);
			this.childNodes.push(newElement);
			return newElement;
		},
		getElementById: function(id) {
			for (var i=0; i<this.childNodes.length; i += 1) {
				if (this.childNodes[i].id === id) {
					return this.childNodes[i];
				}
			}
			return null;
		},
		body: mockHtmlElement("body"),
		head: mockHtmlElement("head"),
		title: ""
	};
}
